<?php
/**
 * The general settings page.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage;

use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\GeneralOptions;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\GeneralOptions\AccessKey;
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for rendering the general settings page.
 */
final class General {
	private const OPTION_GROUP = Plugin::SLUG . '_general_options_group';
	private const SECTION      = Plugin::SLUG . '_general_section';
	public const SUB_PAGE_NAME = Plugin::SLUG;

	/**
	 * This function adds the main menu.
	 */
	public static function add_submenu(): void {
		add_submenu_page(
			parent_slug: SettingsPage::MAIN_PAGE,
			page_title: __( 'General', 'default' ),
			menu_title: __( 'General', 'default' ),
			capability: 'manage_options',
			menu_slug: self::SUB_PAGE_NAME
		);

		do_action( 'locrmaps_after_added_general_submenu' );
	}

	/**
	 * This function registers the settings.
	 */
	public static function register_settings(): void {
		register_setting(
			self::OPTION_GROUP,
			GeneralOptions::OPTION_NAME,
			array(
				'sanitize_callback' => array( self::class, 'sanitize_settings' ),
			)
		);

		add_settings_section(
			id: self::SECTION,
			title: __( 'General', 'default' ),
			callback: array( self::class, 'settings_section_callback' ),
			page: self::SUB_PAGE_NAME
		);

		add_settings_field(
			id: AccessKey::OPTION_KEY,
			title: __( 'Access Key', 'locrmaps-textdomain' ) . ':',
			callback: array( self::class, 'settings_field_callback_for_access_key' ),
			page: self::SUB_PAGE_NAME,
			section: self::SECTION,
			args: array( 'label_for' => AccessKey::OPTION_KEY )
		);
	}

	/**
	 * This function renders the general settings page.
	 */
	public static function render_settings_page(): void {
		print '<div class="wrap">' . PHP_EOL;

		settings_errors();

		print '<h1>' . esc_html__( 'locrMAPS', 'locrmaps-textdomain' ) . ' &rsaquo; ' . esc_html__( 'General', 'default' ) . '</h1>' . PHP_EOL;
		print '<form action="options.php" method="post" name="' . esc_attr( self::SECTION ) . '">' . PHP_EOL;

		settings_fields( self::OPTION_GROUP );
		do_settings_sections( self::SUB_PAGE_NAME );
		submit_button();

		print '</form>' . PHP_EOL;
		print '</div>' . PHP_EOL;
	}

	/**
	 * This function sanitizes the settings.
	 *
	 * @param array<mixed> $input The input.
	 * @return array<mixed> The sanitized input.
	 */
	public static function sanitize_settings( $input ): array {
		if ( isset( $input[ AccessKey::OPTION_KEY ] ) && is_string( $input[ AccessKey::OPTION_KEY ] ) ) {
			try {
				new AccessKey( $input[ AccessKey::OPTION_KEY ] );
			} catch ( \Exception $e ) {
				add_settings_error(
					setting: GeneralOptions::OPTION_NAME,
					code: AccessKey::OPTION_KEY,
					message: $e->getMessage(),
					type: 'error'
				);

				$options = get_option( GeneralOptions::OPTION_NAME );
				if (
					is_array( $options ) &&
					isset( $options[ AccessKey::OPTION_KEY ] ) &&
					is_string( $options[ AccessKey::OPTION_KEY ] )
				) {
					$input[ AccessKey::OPTION_KEY ] = $options[ AccessKey::OPTION_KEY ];
				}
			}
		}

		return $input;
	}

	/**
	 * This function renders the access key field.
	 */
	public static function settings_field_callback_for_access_key(): void {
		$access_key = '';

		try {
			$options    = new GeneralOptions( only_option: AccessKey::OPTION_KEY );
			$access_key = (string) $options->access_key;
		} catch ( \Exception $e ) {
			add_settings_error(
				setting: GeneralOptions::OPTION_NAME,
				code: AccessKey::OPTION_KEY,
				message: $e->getMessage(),
				type: 'error'
			);
		}

		print '<input type="text" class="regular-text" name="' . esc_attr( GeneralOptions::OPTION_NAME . '[' . AccessKey::OPTION_KEY . ']' ) . '" id="' . esc_attr( AccessKey::OPTION_KEY ) . '" value="' . esc_attr( $access_key ) . '">';
	}

	/**
	 * This function renders the section.
	 */
	public static function settings_section_callback(): void {
		printf(
			'<p>%s</p>' . PHP_EOL,
			__( // @phpcs:ignore
				'Please <a href="https://maps.locr.com/en/request-access-key" target="_blank">contact the LOCR team</a> to obtain a valid access key.',
				'locrmaps-textdomain'
			)
		);
	}
}
